/* ============================================================================
 * File Name: Nextion_encoder_02a
 *
 * Description:
 *   Menu example using 3.2” Nextion(*) HMI touch-screen display and rotary encoder.
 *
 *   Full-duplex communication between PSoC and Nextion display established using
 *   serial UART interface. Menu selection is done through the touch-screen,
 *   while encoder utilized to modify the field value. USB-UART interface
 *   (through the KitProg) allows for monitoring of communication between PSoC and
 *   display using serial terminal (RealTerm). 
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#include <project.h>
#include <stdlib.h> //atof
#include <stdio.h>  //spintf
#include <Nextion_Rx.h> 
#include <Nextion_Tx.h>


//============================================
// Global variables
//============================================
char buff[64];//output UART buffer

const char * enum_txt[] = { "0.001", "0.010", "0.100", "1.000", "10.000", "100.000", "1000.000", "10000.000" }; // _enum type txt

//============================================
// Forward declarations
//============================================
void SetupDecoder();
void ProcessNextionMsg(void);      // Process received command (do something...)




void Initialize(void)
{   
    CyGlobalIntEnable;       // enable global interrupts.
    
    Rx_Int_StartEx(MyRxInt); // start Rx interrupt
    
    CyDelay(20);
    
    UART_1_Start();          // Nextion LCD
   
    
    UART_2_Start();         // debugging terminal
    UART_2_PutString("\nNextion LCD menu demo\r\n\n"); 

    
    //setup menu->
    //strcpy( Menu[0].name, "FREQ" );
    Menu[0].type=_float; //float
    Menu[0].value= 1000.0;
    Menu[0].step=0.1;
    Menu[0].min=10;
    Menu[0].max=100000;
    Menu[0].bounds=true;
    
    //strcpy( Menu[1].name, "STEP" );
    Menu[1].type=_enum; // enum
    Menu[1].value=2;    // "0.100"
    Menu[1].step=1;
    Menu[1].min=0;      // "0.001"
    Menu[1].max=7;      // "10000.000"
    Menu[1].bounds=true;
    
    //strcpy( Menu[2].name, "PHSE" );
    Menu[2].type=_int;  // int
    Menu[2].value=0;
    Menu[2].step=4;
    Menu[2].min=-128;
    Menu[2].max=127;
    Menu[2].bounds=true;
    
    //strcpy( Menu[2].name, "AMPL" );
    Menu[3].type=_int;  // int
    Menu[3].value=100;
    Menu[3].step=5;
    Menu[3].min=0;
    Menu[3].max=255;
    Menu[3].bounds=true;
    
      
    Decoder_1_Start();
    
    
    // seems some Nextion bug, must reset twice->
    UART_1_PutString("rest");   // reset Nextion LCD (does not work if reset button not pressed?)
    UART_1_PutString(term_str);
 
    UART_1_PutString("rest");   // reset Nextion LCD (does not work if reset button not pressed?)
    UART_1_PutString(term_str);
    CyDelay(200);               // yield time to refresh LCD 

    //menu_idx = 0;             // select default menu item
    //SetupDecoder();           // fill Decoder fields according to menu_idx (must initialize else erroneous values on 0-item)

    menu_initialized=false;     // no valid menu_idx received yet
    Update_Menu();              // refresh Nextion text fields
    
}


int main()
{
    
    Initialize();
     
    
    for(;;) 
    {
        
        // check for messages from Nextion        
        if(IsCharReady())               // Rx buffer not empty
        {
            if (GetRxStr())             // Some message available
            {
                ProcessNextionMsg();    // execute command
            }
        } 
        
        
        // rotary encoder event                                 
        if (Decoder_1_PositionChanged)  // edit parameter field
        { 
            //BlinkLED1();              // debug..
            
            if (menu_initialized)       // some Nextion component is selected for editing
            //if ( (menu_initialized) && (Decoder_1_Direction!=0) ) // avoid software initialization event
            {
                Menu[menu_idx].value = Decoder_1_Position; // new value
                Update_Value(menu_idx); // update edited field
                
                // take action here... (update PWM, etc.)
            }    
        }
        
        /*
        if (Decoder_1_BtnPressed)
        {
            // button switch not used in this demo
        }
        */
        
    }


    
} //main

//===========================================================================
// Process message received in Receive Buffer from Nextion HMI:
// RB.RxStr - array of char received
// RB_len   - length of array (including terminator {0xFF,0xFF,0xFF}
//
// check received message for valid comonent touch and execute
// if command not recognized, then show content on debug screen 
//===========================================================================

void ProcessNextionMsg(void)
{    
    
    if (RB.Event == 0x65) { // Nextion touch event (0x65)
        
        menu_initialized=true;
        
        // lookup table: 'Nextion Component ID' -> 'Menu index' 
        switch (RB.cID) {
        case 0x01: menu_idx = 0;
            break;
        case 0x02: menu_idx = 1;
            break;
        case 0x03: menu_idx = 2;
            break;
        case 0x04: menu_idx = 3;
            break;
        default: 
            menu_initialized=false; // menu_idx is not valid
        }      
        
        SetupDecoder();             // setup Decoder with parameters from Menu[menu_idx]
    }
    else                            //command unrecognized - echo in Hex format on debug terminal
    {
        UART_2_PutString("unrecognized message:");
        uint8 i;
        for (i=0; i<Rx_len; i++) {
            sprintf(buff,"%X ", RB.RxStr[i]); UART_2_PutString(buff);
        }
        UART_2_PutCRLF(0);
    }
    
}

//==============================================================================
// Update Decoder_1 settings according to Menu current state:
// 'main menu' or 'editing a value'
//==============================================================================

void SetupDecoder()
{
    TMenuItem Item = Menu[menu_idx]; // entering menu item #menu_idx
    
    // modify Frequency.step (index=0) according to Increment.value (index=1)
    if (menu_idx==0) {
        Item.step=atof(enum_txt[ (uint8) Menu[1].value ]); // e.g "0.010" -> 0.01
    }
        
    Decoder_1_Setup( Item.value, Item.step, Item.min, Item.max, Item.bounds ); 
}


/// END OF PROGRAM //////////////////////////////////////////////////////////////////////////////////////
