/* ============================================================================
 * File Name: encoder_poll_exN
 *
 * Description:
 *   Example of multiple Decoders operating in the polling mode
 *   using a single EXTERNAL polling interrupt.
 *
 *   This mode of operation saves interrupts and can useful e.g. for PSoC4
 *   where the number of available interrupts is limited. All encoders are sampled
 *   at 4 kHz rate on clock interrupt, and a flag is raised if position change has
 *   been detected. Note that the clock input on component is hidden in this mode.
 *   Pressing a button resets encoder position. USBUART is used to output position
 *   and direction of rotation on the serial terminal. 
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#include <project.h>
#include <stdlib.h>
#include <stdio.h>
#include <USBUART.h>


//============================================
// Global variables
//============================================
char buff[64];//output UART buffer

//============================================
// interrupt on encoder pin states change
//============================================
volatile CYBIT flag_DataReady = 0; // semaphore flag
CY_ISR(myDtrInt) 
{   
    flag_DataReady = 1;
}


//============================================
// Forward declarations
//============================================
void DisplayData(void); // output to terminal




void Initialize(void)
{   
    CyGlobalIntEnable;          //enable global interrupts.
    
    USBUART_Init();             // initialize USBUART
    
    sprintf(buff, "\n%s", "Rotary encoder example\r\n");
    USBUART_1_PutString(buff);  // Send data to terminal
    
    DisplayData();
    
    
    Decoder_1_Start();
    Decoder_2_Start();
    Decoder_3_Start();
    Decoder_4_Start();
    Decoder_5_Start();
    Decoder_6_Start();
    Decoder_7_Start();
    Decoder_8_Start();
    
    
    isrDtr_StartEx(myDtrInt);   //start isr_Dtr interrupt
}


int main()
{
    
    Initialize();
     
    
    for(;;) 
    {
        
        // to operate correctly, isr_Poll inside component must be disabled
        if (flag_DataReady==1)          // poll encoders state on timer
        {
            flag_DataReady=0;          
            Decoder_1_CheckStatus();
            Decoder_2_CheckStatus();
            Decoder_3_CheckStatus();
            Decoder_4_CheckStatus();
            Decoder_5_CheckStatus();
            Decoder_6_CheckStatus();
            Decoder_7_CheckStatus();
            Decoder_8_CheckStatus();
        }   
        
        
        if (Decoder_1_PositionChanged) DisplayData(); // output to terminal       
        if (Decoder_2_PositionChanged) DisplayData();        
        if (Decoder_3_PositionChanged) DisplayData();        
        if (Decoder_4_PositionChanged) DisplayData();        
        if (Decoder_5_PositionChanged) DisplayData();        
        if (Decoder_6_PositionChanged) DisplayData();        
        if (Decoder_7_PositionChanged) DisplayData();        
        if (Decoder_8_PositionChanged) DisplayData();        

        if (Decoder_1_BtnPressed) Decoder_1_SetPosition(0); // some action
        if (Decoder_2_BtnPressed) Decoder_2_SetPosition(0);   
        if (Decoder_3_BtnPressed) Decoder_3_SetPosition(0);   
        if (Decoder_4_BtnPressed) Decoder_4_SetPosition(0);   
        if (Decoder_5_BtnPressed) Decoder_5_SetPosition(0);   
        if (Decoder_6_BtnPressed) Decoder_6_SetPosition(0);  
        if (Decoder_7_BtnPressed) Decoder_7_SetPosition(0); 
        if (Decoder_8_BtnPressed) Decoder_8_SetPosition(0); 
                
    }

} //main



//==============================================================================
// 
//==============================================================================

void DisplayData(void) // output to terminal
{ 
    
    sprintf(buff, "%d \t%d \t%d \t%d \r\n", Decoder_1_Position, Decoder_1_Direction, Decoder_2_Position, Decoder_2_Direction ); // ok: int16, uint16
    
    /*
    if(USBUART_1_GetConfiguration() != 0u)      // Service USB CDC when device configured
    { 
        while(USBUART_1_CDCIsReady() == 0u);    // Wait for USB-UART is ready to send more data to the PC (no effect here)  
        USBUART_1_PutString(buff);              // Send data to PC
    }
    */
    
    
    if (USBUART_1_CDCIsReady())
        USBUART_1_PutString(buff);              // Send data to PC
      
    //uint32 ctr = 0;
    //while(USBUART_1_CDCIsReady() == 0u) {ctr++;  if (ctr > 100000) return; } // prevent buffer overflow if Multichart stopped
    //USBUART_1_PutString(buff);              // Send data to PC

}





/// END OF PROGRAM //////////////////////////////////////////////////////////////////////////////////////
